from ..database import Database
from PyQt5.QtWidgets import QWidget


class _Window(QWidget):
    """The base class for every qt specific window"""

    def __init__(self, window):
        super().__init__()
        self.window = window
        self.setupUi(self.window)
        self.after_setup_ui()

    def setupUi(self, window) -> None:
        """setupUi method which creates the basic window, generated from a .ui file with the pyuic5 command line tool"""
        pass

    def after_setup_ui(self) -> None:
        """
        Here should stay every ui specific code which should be executed before opening the window and after calling `self.setupUi`.
        This is to prevent to overload the `self.__init__` method
        """
        pass

    def show(self) -> None:
        """Show / executes the window"""
        try:
            self.window.exec()
        except AttributeError:
            self.window.show()

    def close(self) -> None:
        """Closes the window"""
        self.window.close()


class _DatabaseWindow(_Window):
    """Base class for every window besides `window.Window` which want to use the database"""

    def __init__(self, window):
        # the import is here, because if it would be imported above, a circular import would be created
        from .window import _database

        if not _database:
            raise IOError('Database or main window is not initialized')
        else:
            self.database: Database = _database

        super().__init__(window)
