from . import _DatabaseWindow
from .. import config
from ..database import default_databases, Database
from PyQt5 import QtCore, QtGui, QtWidgets, QtSql
import os
from copy import copy
from . import style
from typing import Union


class Connect(_DatabaseWindow):
    """Class to connect to another database"""

    def __init__(self, window):
        self.file = ''
        self.database = None
        self.current_index = -1
        self.is_database_file_new = False

        super().__init__(window)

        self.on_database_index_change(self.database_type.currentIndex())

    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(400, 490)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Form.sizePolicy().hasHeightForWidth())
        Form.setSizePolicy(sizePolicy)
        self.verticalLayout = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout.setSizeConstraint(QtWidgets.QLayout.SetMinimumSize)
        self.verticalLayout.setObjectName("verticalLayout")
        self.database_type_hlayout = QtWidgets.QHBoxLayout()
        self.database_type_hlayout.setObjectName("database_type_hlayout")
        self.database_label = QtWidgets.QLabel(Form)
        self.database_label.setObjectName("database_label")
        self.database_type_hlayout.addWidget(self.database_label)
        spacerItem = QtWidgets.QSpacerItem(13, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.database_type_hlayout.addItem(spacerItem)
        self.database_type = QtWidgets.QComboBox(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.database_type.sizePolicy().hasHeightForWidth())
        self.database_type.setSizePolicy(sizePolicy)
        self.database_type.setObjectName("database_type")
        self.database_type_hlayout.addWidget(self.database_type)
        self.verticalLayout.addLayout(self.database_type_hlayout)
        self.line = QtWidgets.QFrame(Form)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.database_file_hlayout = QtWidgets.QHBoxLayout()
        self.database_file_hlayout.setObjectName("database_file_hlayout")
        self.database_file = QtWidgets.QPushButton(Form)
        self.database_file.setMinimumSize(QtCore.QSize(118, 0))
        self.database_file.setMaximumSize(QtCore.QSize(130, 16777215))
        self.database_file.setObjectName("database_file")
        self.database_file_hlayout.addWidget(self.database_file)
        self.file_name_label = QtWidgets.QLabel(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.file_name_label.sizePolicy().hasHeightForWidth())
        self.file_name_label.setSizePolicy(sizePolicy)
        self.file_name_label.setObjectName("file_name_label")
        self.database_file_hlayout.addWidget(self.file_name_label)
        self.file_utils_hlayout = QtWidgets.QHBoxLayout()
        self.file_utils_hlayout.setSpacing(0)
        self.file_utils_hlayout.setObjectName("file_utils_hlayout")
        self.line_2 = QtWidgets.QFrame(Form)
        self.line_2.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.file_utils_hlayout.addWidget(self.line_2)
        self.delete_file_button = QtWidgets.QPushButton(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.delete_file_button.sizePolicy().hasHeightForWidth())
        self.delete_file_button.setSizePolicy(sizePolicy)
        self.delete_file_button.setStyleSheet("")
        self.delete_file_button.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/file_url/trash"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.delete_file_button.setIcon(icon)
        self.delete_file_button.setIconSize(QtCore.QSize(18, 18))
        self.delete_file_button.setAutoDefault(False)
        self.delete_file_button.setDefault(False)
        self.delete_file_button.setFlat(True)
        self.delete_file_button.setObjectName("delete_file_button")
        self.file_utils_hlayout.addWidget(self.delete_file_button)
        self.add_file_button = QtWidgets.QPushButton(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.add_file_button.sizePolicy().hasHeightForWidth())
        self.add_file_button.setSizePolicy(sizePolicy)
        self.add_file_button.setStyleSheet("")
        self.add_file_button.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/file_url/add_file"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.add_file_button.setIcon(icon1)
        self.add_file_button.setIconSize(QtCore.QSize(18, 18))
        self.add_file_button.setFlat(True)
        self.add_file_button.setObjectName("add_file_button")
        self.file_utils_hlayout.addWidget(self.add_file_button)
        self.database_file_hlayout.addLayout(self.file_utils_hlayout)
        self.verticalLayout.addLayout(self.database_file_hlayout)
        self.database_settings_frame = QtWidgets.QFrame(Form)
        self.database_settings_frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.database_settings_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.database_settings_frame.setObjectName("database_settings_frame")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.database_settings_frame)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.database_ip_layout = QtWidgets.QHBoxLayout()
        self.database_ip_layout.setObjectName("database_ip_layout")
        self.database_ip_label = QtWidgets.QLabel(self.database_settings_frame)
        self.database_ip_label.setMinimumSize(QtCore.QSize(100, 0))
        self.database_ip_label.setObjectName("database_ip_label")
        self.database_ip_layout.addWidget(self.database_ip_label)
        self.database_ip_edit = QtWidgets.QLineEdit(self.database_settings_frame)
        self.database_ip_edit.setObjectName("database_ip_edit")
        self.database_ip_layout.addWidget(self.database_ip_edit)
        self.verticalLayout_2.addLayout(self.database_ip_layout)
        self.user_hlayout = QtWidgets.QHBoxLayout()
        self.user_hlayout.setObjectName("user_hlayout")
        self.user_label = QtWidgets.QLabel(self.database_settings_frame)
        self.user_label.setMinimumSize(QtCore.QSize(100, 0))
        self.user_label.setObjectName("user_label")
        self.user_hlayout.addWidget(self.user_label)
        self.user_edit = QtWidgets.QLineEdit(self.database_settings_frame)
        self.user_edit.setObjectName("user_edit")
        self.user_hlayout.addWidget(self.user_edit)
        self.verticalLayout_2.addLayout(self.user_hlayout)
        self.password_hlayout = QtWidgets.QHBoxLayout()
        self.password_hlayout.setObjectName("password_hlayout")
        self.password_label = QtWidgets.QLabel(self.database_settings_frame)
        self.password_label.setMinimumSize(QtCore.QSize(100, 0))
        self.password_label.setObjectName("password_label")
        self.password_hlayout.addWidget(self.password_label)
        self.password_edit = QtWidgets.QLineEdit(self.database_settings_frame)
        self.password_edit.setInputMethodHints(QtCore.Qt.ImhHiddenText|QtCore.Qt.ImhNoAutoUppercase|QtCore.Qt.ImhNoPredictiveText|QtCore.Qt.ImhSensitiveData)
        self.password_edit.setEchoMode(QtWidgets.QLineEdit.Password)
        self.password_edit.setObjectName("password_edit")
        self.password_hlayout.addWidget(self.password_edit)
        self.verticalLayout_2.addLayout(self.password_hlayout)
        self.port_hlayout = QtWidgets.QHBoxLayout()
        self.port_hlayout.setObjectName("port_hlayout")
        self.port_label = QtWidgets.QLabel(self.database_settings_frame)
        self.port_label.setMinimumSize(QtCore.QSize(100, 0))
        self.port_label.setObjectName("port_label")
        self.port_hlayout.addWidget(self.port_label)
        self.port_edit = QtWidgets.QLineEdit(self.database_settings_frame)
        self.port_edit.setFrame(True)
        self.port_edit.setClearButtonEnabled(False)
        self.port_edit.setObjectName("port_edit")
        self.port_hlayout.addWidget(self.port_edit)
        self.verticalLayout_2.addLayout(self.port_hlayout)
        self.database_name_hlayout = QtWidgets.QHBoxLayout()
        self.database_name_hlayout.setObjectName("database_name_hlayout")
        self.database_name_label = QtWidgets.QLabel(self.database_settings_frame)
        self.database_name_label.setMinimumSize(QtCore.QSize(100, 0))
        self.database_name_label.setObjectName("database_name_label")
        self.database_name_hlayout.addWidget(self.database_name_label)
        self.database_name_edit = QtWidgets.QLineEdit(self.database_settings_frame)
        self.database_name_edit.setPlaceholderText("")
        self.database_name_edit.setObjectName("database_name_edit")
        self.database_name_hlayout.addWidget(self.database_name_edit)
        self.verticalLayout_2.addLayout(self.database_name_hlayout)
        self.show_databases_button = QtWidgets.QPushButton(self.database_settings_frame)
        self.show_databases_button.setObjectName("show_databases_button")
        self.verticalLayout_2.addWidget(self.show_databases_button)
        self.databases_view = QtWidgets.QListView(self.database_settings_frame)
        self.databases_view.setInputMethodHints(QtCore.Qt.ImhNone)
        self.databases_view.setObjectName("databases_view")
        self.verticalLayout_2.addWidget(self.databases_view)
        self.verticalLayout.addWidget(self.database_settings_frame)
        self.test_save_hlayout = QtWidgets.QHBoxLayout()
        self.test_save_hlayout.setObjectName("test_save_hlayout")
        self.test_connection_button = QtWidgets.QPushButton(Form)
        self.test_connection_button.setToolTip("")
        self.test_connection_button.setStatusTip("")
        self.test_connection_button.setWhatsThis("")
        self.test_connection_button.setObjectName("test_connection_button")
        self.test_save_hlayout.addWidget(self.test_connection_button)
        self.save_button = QtWidgets.QPushButton(Form)
        self.save_button.setObjectName("save_button")
        self.test_save_hlayout.addWidget(self.save_button)
        self.verticalLayout.addLayout(self.test_save_hlayout)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Connect to database"))
        self.database_label.setText(_translate("Form", "Database"))
        self.database_file.setText(_translate("Form", "Database file"))
        self.file_name_label.setText(_translate("Form", "No file selected"))
        self.database_ip_label.setText(_translate("Form", "Database IP"))
        self.database_ip_edit.setText(_translate("Form", "127.0.0.1"))
        self.user_label.setText(_translate("Form", "User"))
        self.user_edit.setText(_translate("Form", "root"))
        self.password_label.setText(_translate("Form", "Password"))
        self.port_label.setText(_translate("Form", "Port"))
        self.port_edit.setText(_translate("Form", "3306"))
        self.database_name_label.setText(_translate("Form", "Database name"))
        self.database_name_edit.setText(_translate("Form", "athnos"))
        self.show_databases_button.setText(_translate("Form", "Show available names"))
        self.test_connection_button.setText(_translate("Form", "Test connection"))
        self.save_button.setText(_translate("Form", "Save"))

    def after_setup_ui(self) -> None:
        item_model = QtGui.QStandardItemModel()
        # loads all sql driver to the database combobox
        for i, driver in enumerate(default_databases):
            item = QtGui.QStandardItem(driver['title'])
            # if the driver is not available, the index gets disabled
            if not QtSql.QSqlDatabase.isDriverAvailable(driver['driver']):
                item.setEnabled(False)
            item_model.setItem(i, 0, item)

        # sets the item model to the database type combobox
        self.database_type.setModel(item_model)

        # gets called if another database driver is chosen
        self.database_type.currentIndexChanged.connect(self.on_database_index_change)

        # database file specific
        self.database_file.clicked.connect(self.on_choose_database_file)
        self.delete_file_button.clicked.connect(self.on_remove_database_file)
        self.add_file_button.clicked.connect(self.on_new_database_file)

        # database settings
        self.port_edit.setValidator(QtGui.QIntValidator(0, 65535))

        self.show_databases_button.clicked.connect(self.on_show_databases)
        self.database_name_proxy_model = QtCore.QSortFilterProxyModel()
        self.database_name_edit.textChanged.connect(lambda: self.database_name_proxy_model.setFilterRegExp(QtCore.QRegExp(f'^{self.database_name_edit.text()}')) if not self.databases_view.isHidden() else None)
        self.databases_view.doubleClicked.connect(lambda a0: self.database_name_edit.setText(a0.data(QtCore.Qt.DisplayRole)))

        # test connection / save buttons
        self.test_connection_button.clicked.connect(self.on_test_connection)
        self.save_button.clicked.connect(self.on_save)

        # chooses the default database
        self.on_database_index_change(-1 if len(default_databases) == 0 else 0)

    def on_database_index_change(self, index: int) -> None:
        """Changes the text edit etc. settings, depending on the `index`"""
        if index == -1:
            self.database_settings_frame.setEnabled(False)
            self.database_ip_edit.setText('')
            self.database_name_edit.setText('')
            self.user_edit.setText('')
            self.password_edit.setText('')
            self.port_edit.setText('')
            return

        self.current_index = index
        database = default_databases[index]

        # sets the content of `self.database_file_hlayout` enabled or disabled, according to if database['file'] is True or False
        for i in range(self.database_file_hlayout.count()):
            item = self.database_file_hlayout.itemAt(i)
            try:
                item.widget().setEnabled(database['ip'] is None)
            except AttributeError:
                for j in range(item.layout().count()):
                    item.layout().itemAt(j).widget().setEnabled(database['ip'] is None)

        self.databases_view.setModel(None)

        # currently only sqlite3
        if database['ip'] is None:
            self.file_name_label.setText('No file selected')
            self.database_settings_frame.setEnabled(False)
            self.database_ip_edit.setText('')
            self.database_name_edit.setText('')
            self.user_edit.setText('')
            self.password_edit.setText('')
            self.port_edit.setText('')
        else:
            self.file_name_label.setText('No file selected')
            self.database_settings_frame.setEnabled(True)
            self.database_ip_edit.setText('127.0.0.1')
            self.database_name_edit.setText('athnos')
            self.user_edit.setText(database['user'])
            self.password_edit.setText(database['password'])
            self.port_edit.setText(str(database['port']))

    def on_choose_database_file(self) -> None:
        """Opens a file choose dialog"""
        if default_databases[self.current_index]['driver'] == 'QSQLITE':
            file_name = 'SQLite3 database'
            file_endings = ['sqlite', 'sqlite3']
        else:
            file_endings = ['*']
            file_name = 'Database file'

        file = QtWidgets.QFileDialog.getOpenFileName(self.window, filter=f'{file_name}({" *.".join(file_endings)});;All files(*.*)')[0]
        if file:
            self.file = file
            self.file_name_label.setText(file)

    def on_new_database_file(self) -> None:
        """Opens a create file dialog for a database file"""
        if default_databases[self.current_index]['driver'] == 'QSQLITE':
            file_name = 'SQLite3 database'
            file_endings = ['sqlite', 'sqlite3']
        else:
            file_endings = ['*']
            file_name = 'Database file'

        file = QtWidgets.QFileDialog.getSaveFileName(self.window, filter=f'{file_name}({" *.".join(file_endings)});;All files(*.*)')[0]
        if file:
            self.file = file
            self.file_name_label.setText(file)
            self.is_database_file_new = True

    def on_remove_database_file(self) -> None:
        """Removes the database file (internal)"""
        self.file = None
        self.file_name_label.setText('No file selected')

    def on_show_databases(self) -> None:
        """Shows all available databases (names)"""
        if self.database_type.currentText().lower() in ('mariadb', 'mysql', 'postgresql'):
            result = QtWidgets.QMessageBox.question(self.window, 'Connect to database',
                                                    'To show all databases / database names, I have to connect with the ip. '
                                                    'Proceed?')
            if result == QtWidgets.QMessageBox.Yes:
                databases_model = QtGui.QStandardItemModel()
                db = self.on_test_connection(True, False)
                if db:
                    if self.database_type.currentText().lower() == 'postgresql':
                        databases = db.exec('SELECT datname FROM pg_database')
                    else:
                        databases = db.exec('SHOW DATABASES')
                    i = 0
                    while databases.next():
                        databases_model.setItem(i, 0, QtGui.QStandardItem(databases.value(0)))
                        i += 1

                    self.database_name_proxy_model.setSourceModel(databases_model)
                    self.databases_view.setModel(self.database_name_proxy_model)
                    self.database_name_edit.setText('')
        else:
            QtWidgets.QMessageBox.warning(self.window, 'Not supported', 'Currently database / name detection is only available for '
                                                                        'MariaDB, MySQL and PostgreSQL')

    def on_test_connection(self, return_db=False, specify_database=True) -> Union[QtSql.QSqlDatabase, None]:
        """Test the connection to the database with the current settings"""
        database = default_databases[self.current_index]

        db = QtSql.QSqlDatabase(database['driver'])

        # only sqlite3 which uses files to store the data in it has no ip
        if not database['ip']:
            if self.file:
                if os.path.isfile(self.file) or self.is_database_file_new:
                    db.setDatabaseName(self.file)
                    message = None
                else:
                    message = f'The file {self.file} does not exist'
            else:
                message = 'No file is selected'

            if message:
                # the connection would be successful, even if the file does not exist
                QtWidgets.QMessageBox.warning(self.window, 'Connection failed', f'Could not connect to the database: {message}',
                                              QtWidgets.QMessageBox.Ok)
                return
        else:
            if specify_database:
                db.setDatabaseName(self.database_name_edit.text())

            db.setHostName(self.database_ip_edit.text())
            db.setUserName(self.user_edit.text())
            db.setPassword(self.password_edit.text())
            db.setPort(int(self.port_edit.text()))

        if db.open():
            if specify_database:
                is_table = 'test' in db.tables()
                # creates a test table to check if the database integrity
                db.exec('CREATE TABLE IF NOT EXISTS test (test varchar(1))')
            if not db.lastError().isValid():
                if specify_database and not is_table:
                    db.exec('DROP TABLE test')
                if return_db:
                    return db
                else:
                    QtWidgets.QMessageBox.information(self.window, 'Connection successful', 'Connected successful to the database', QtWidgets.QMessageBox.Ok)
                    db.close()
                    return
        QtWidgets.QMessageBox.warning(self.window, 'Connection failed', f'Could not connect to the database: {db.lastError().text()}', QtWidgets.QMessageBox.Ok)

        return

    def on_save(self) -> None:
        """Saves the connection"""
        # the connection gets tested first
        db = self.on_test_connection(True)
        if db:
            self.database = Database(db, self.database._auto_table)
            # writes the connection in the config
            for item in default_databases:
                if item['driver'] == db.driverName():
                    item = copy(item)
                    if self.file:
                        item['name'] = self.file
                    else:
                        item['name'] = self.database_ip_edit.text()
                    config['database'] = item
                    break

            self.close()

    def show(self) -> Union[QtSql.QSqlDatabase, None]:
        super().show()
        return self.database
